@@ -2,7 +2,7 @@ mod allocator;
 pub mod vulkan;
 
 use crate::RendererError;
-use ash::{version::DeviceV1_0, vk, Device, Instance};
+use ash::{vk, Device};
 use imgui::{Context, DrawCmd, DrawCmdParams, DrawData, TextureId, Textures};
 use mesh::*;
 use ultraviolet::projection::orthographic_vk;
@@ -10,70 +10,46 @@ use vulkan::*;
 
 use self::allocator::Allocator;
 
+#[cfg(not(any(feature = "gpu-allocator", feature = "vk-mem")))]
+use ash::Instance;
+
+#[cfg(feature = "gpu-allocator")]
+use {
+    gpu_allocator::vulkan::Allocator as GpuAllocator,
+    std::sync::{Arc, Mutex},
+};
+
+#[cfg(feature = "vk-mem")]
+use {
+    std::sync::{Arc, Mutex},
+    vk_mem::Allocator as VkMemAllocator,
+};
+
 /// Convenient return type for function that can return a [`RendererError`].
 ///
 /// [`RendererError`]: enum.RendererError.html
 pub type RendererResult<T> = Result<T, RendererError>;
 
-/// Trait providing access to the application's Vulkan context.
-pub trait RendererVkContext {
-    /// Return a reference to the Vulkan instance.
-    fn instance(&self) -> &Instance;
-
-    /// Return the Vulkan physical device.
-    fn physical_device(&self) -> vk::PhysicalDevice;
-
-    /// Return a reference to the Vulkan device.
-    fn device(&self) -> &Device;
-
-    /// Return a Vulkan queue.
-    ///
-    /// It will be used to submit commands during initialization to upload
-    /// data to the gpu. The type of queue must be supported by the following
-    /// commands:
-    ///
-    /// * [vkCmdCopyBufferToImage](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdCopyBufferToImage.html)
-    /// * [vkCmdPipelineBarrier](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdPipelineBarrier.html)
-    fn queue(&self) -> vk::Queue;
-
-    /// Return a Vulkan command pool.
-    ///
-    /// The pool will be used to allocate command buffers to upload textures to the gpu.
-    fn command_pool(&self) -> vk::CommandPool;
-
-    /// Return a reference to a vk-mem Allocator.
-    ///
-    /// The allocator will be used to allocate images and buffers.
-    #[cfg(feature = "vkmem")]
-    fn vk_mem_allocator(&self) -> &vk_mem::Allocator;
-}
-
 /// Vulkan renderer for imgui.
 ///
 /// It records rendering command to the provided command buffer at each call to [`cmd_draw`].
-/// When done with the renderer you should call [`destroy`] before droping it to release all
-/// Vulkan resources held by the renderer.
-///
-/// All methods take a reference to a type implementing the [`RendererVkContext`] trait.
 ///
 /// The renderer holds a set of vertex/index buffers per in flight frames. Vertex and index buffers
 /// are resized at each call to [`cmd_draw`] if draw data does not fit.
 ///
 /// [`cmd_draw`]: #method.cmd_draw
-/// [`destroy`]: #method.destroy
-/// [`RendererVkContext`]: trait.RendererVkContext.html
 pub struct Renderer {
+    device: Device,
     allocator: Allocator,
     pipeline: vk::Pipeline,
     pipeline_layout: vk::PipelineLayout,
     descriptor_set_layout: vk::DescriptorSetLayout,
-    fonts_texture: Texture,
+    fonts_texture: Option<Texture>,
     descriptor_pool: vk::DescriptorPool,
     descriptor_set: vk::DescriptorSet,
     textures: Textures<vk::DescriptorSet>,
     in_flight_frames: usize,
     frames: Option<Frames>,
-    destroyed: bool,
 }
 
 impl Renderer {
@@ -84,7 +60,15 @@ impl Renderer {
     ///
     /// # Arguments
     ///
-    /// * `vk_context` - A reference to a type implementing the [`RendererVkContext`] trait.
+    /// * `instance` - A reference to a Vulkan instance.
+    /// * `physical_device` - A Vulkan physical device.
+    /// * `device` - A Vulkan device.
+    /// * `queue` - A Vulkan queue.
+    ///             It will be used to submit commands during initialization to upload
+    ///             data to the gpu. The type of queue must be supported by the following
+    ///             commands: [vkCmdCopyBufferToImage](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdCopyBufferToImage.html),
+    ///             [vkCmdPipelineBarrier](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdPipelineBarrier.html)
+    /// * `command_pool` - A Vulkan command pool used to allocate command buffers to upload textures to the gpu.
     /// * `in_flight_frames` - The number of in flight frames of the application.
     /// * `render_pass` - The render pass used to render the gui.
     /// * `imgui` - The imgui context.
@@ -93,18 +77,25 @@ impl Renderer {
     ///
     /// * [`RendererError`] - If the number of in flight frame in incorrect.
     /// * [`RendererError`] - If any Vulkan or io error is encountered during initialization.
-    ///
-    /// [`RendererVkContext`]: trait.RendererVkContext.html
-    /// [`RendererError`]: enum.RendererError.html
-    pub fn new<C: RendererVkContext>(
-        vk_context: &C,
+    #[cfg(not(any(feature = "gpu-allocator", feature = "vk-mem")))]
+    pub fn with_default_allocator(
+        instance: &Instance,
+        physical_device: vk::PhysicalDevice,
+        device: Device,
+        queue: vk::Queue,
+        command_pool: vk::CommandPool,
         in_flight_frames: usize,
         render_pass: vk::RenderPass,
         imgui: &mut Context,
     ) -> RendererResult<Self> {
+        let memory_properties =
+            unsafe { instance.get_physical_device_memory_properties(physical_device) };
+
         Self::from_allocator(
-            vk_context,
-            Allocator::defaut(),
+            device,
+            queue,
+            command_pool,
+            Allocator::new(memory_properties),
             in_flight_frames,
             render_pass,
             imgui,
@@ -118,7 +109,14 @@ impl Renderer {
     ///
     /// # Arguments
     ///
-    /// * `vk_context` - A reference to a type implementing the [`RendererVkContext`] trait.
+    /// * `gpu_allocator` - The allocator that will be used to allocator buffer and image memory.
+    /// * `device` - A Vulkan device.
+    /// * `queue` - A Vulkan queue.
+    ///             It will be used to submit commands during initialization to upload
+    ///             data to the gpu. The type of queue must be supported by the following
+    ///             commands: [vkCmdCopyBufferToImage](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdCopyBufferToImage.html),
+    ///             [vkCmdPipelineBarrier](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdPipelineBarrier.html)
+    /// * `command_pool` - A Vulkan command pool used to allocate command buffers to upload textures to the gpu.
     /// * `in_flight_frames` - The number of in flight frames of the application.
     /// * `render_pass` - The render pass used to render the gui.
     /// * `imgui` - The imgui context.
@@ -127,28 +125,76 @@ impl Renderer {
     ///
     /// * [`RendererError`] - If the number of in flight frame in incorrect.
     /// * [`RendererError`] - If any Vulkan or io error is encountered during initialization.
-    ///
-    /// [`RendererVkContext`]: trait.RendererVkContext.html
-    /// [`RendererError`]: enum.RendererError.html
-    #[cfg(feature = "vkmem")]
-    pub fn with_vk_mem_allocator<C: RendererVkContext>(
-        vk_context: &C,
+    #[cfg(feature = "gpu-allocator")]
+    pub fn with_gpu_allocator(
+        gpu_allocator: Arc<Mutex<GpuAllocator>>, // TODO: Another way ?
+        device: Device,
+        queue: vk::Queue,
+        command_pool: vk::CommandPool,
         in_flight_frames: usize,
         render_pass: vk::RenderPass,
         imgui: &mut Context,
     ) -> RendererResult<Self> {
         Self::from_allocator(
-            vk_context,
-            Allocator::vk_mem(),
+            device,
+            queue,
+            command_pool,
+            Allocator::new(gpu_allocator),
             in_flight_frames,
             render_pass,
             imgui,
         )
     }
 
-    fn from_allocator<C: RendererVkContext>(
-        vk_context: &C,
-        allocator: Allocator,
+    /// Initialize and return a new instance of the renderer.
+    ///
+    /// At initialization all Vulkan resources are initialized and font texture is created and
+    /// uploaded to the gpu. Vertex and index buffers are not created yet.
+    ///
+    /// # Arguments
+    ///
+    /// * `vk_mem_allocator` - The allocator that will be used to allocator buffer and image memory.
+    /// * `device` - A Vulkan device.
+    /// * `queue` - A Vulkan queue.
+    ///             It will be used to submit commands during initialization to upload
+    ///             data to the gpu. The type of queue must be supported by the following
+    ///             commands: [vkCmdCopyBufferToImage](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdCopyBufferToImage.html),
+    ///             [vkCmdPipelineBarrier](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdPipelineBarrier.html)
+    /// * `command_pool` - A Vulkan command pool used to allocate command buffers to upload textures to the gpu.
+    /// * `in_flight_frames` - The number of in flight frames of the application.
+    /// * `render_pass` - The render pass used to render the gui.
+    /// * `imgui` - The imgui context.
+    ///
+    /// # Errors
+    ///
+    /// * [`RendererError`] - If the number of in flight frame in incorrect.
+    /// * [`RendererError`] - If any Vulkan or io error is encountered during initialization.
+    #[cfg(feature = "vk-mem")]
+    pub fn with_vk_mem_allocator(
+        vk_mem_allocator: Arc<Mutex<VkMemAllocator>>, // TODO: Another way ?
+        device: Device,
+        queue: vk::Queue,
+        command_pool: vk::CommandPool,
+        in_flight_frames: usize,
+        render_pass: vk::RenderPass,
+        imgui: &mut Context,
+    ) -> RendererResult<Self> {
+        Self::from_allocator(
+            device,
+            queue,
+            command_pool,
+            Allocator::new(vk_mem_allocator),
+            in_flight_frames,
+            render_pass,
+            imgui,
+        )
+    }
+
+    fn from_allocator(
+        device: Device,
+        queue: vk::Queue,
+        command_pool: vk::CommandPool,
+        mut allocator: Allocator,
         in_flight_frames: usize,
         render_pass: vk::RenderPass,
         imgui: &mut Context,
@@ -160,12 +206,11 @@ impl Renderer {
         }
 
         // Descriptor set layout
-        let descriptor_set_layout = create_vulkan_descriptor_set_layout(vk_context.device())?;
+        let descriptor_set_layout = create_vulkan_descriptor_set_layout(&device)?;
 
         // Pipeline and layout
-        let pipeline_layout =
-            create_vulkan_pipeline_layout(vk_context.device(), descriptor_set_layout)?;
-        let pipeline = create_vulkan_pipeline(vk_context.device(), pipeline_layout, render_pass)?;
+        let pipeline_layout = create_vulkan_pipeline_layout(&device, descriptor_set_layout)?;
+        let pipeline = create_vulkan_pipeline(&device, pipeline_layout, render_pass)?;
 
         // Fonts texture
         let fonts_texture = {
@@ -173,11 +218,13 @@ impl Renderer {
             let atlas_texture = fonts.build_rgba32_texture();
 
             Texture::from_rgba8(
-                vk_context,
-                &allocator,
+                &device,
+                queue,
+                command_pool,
+                &mut allocator,
                 atlas_texture.width,
                 atlas_texture.height,
-                &atlas_texture.data,
+                atlas_texture.data,
             )?
         };
 
@@ -185,11 +232,11 @@ impl Renderer {
         fonts.tex_id = TextureId::from(usize::MAX);
 
         // Descriptor pool
-        let descriptor_pool = create_vulkan_descriptor_pool(vk_context.device(), 1)?;
+        let descriptor_pool = create_vulkan_descriptor_pool(&device, 1)?;
 
         // Descriptor set
         let descriptor_set = create_vulkan_descriptor_set(
-            vk_context.device(),
+            &device,
             descriptor_set_layout,
             descriptor_pool,
             fonts_texture.image_view,
@@ -200,17 +247,17 @@ impl Renderer {
         let textures = Textures::new();
 
         Ok(Self {
+            device,
             allocator,
             pipeline,
             pipeline_layout,
             descriptor_set_layout,
-            fonts_texture,
+            fonts_texture: Some(fonts_texture),
             descriptor_pool,
             descriptor_set,
             textures,
             in_flight_frames,
             frames: None,
-            destroyed: false,
         })
     }
 
@@ -222,29 +269,14 @@ impl Renderer {
     ///
     /// # Arguments
     ///
-    /// * `vk_context` - A reference to a type implementing the [`RendererVkContext`] trait.
     /// * `render_pass` - The render pass used to render the gui.
     ///
     /// # Errors
     ///
     /// * [`RendererError`] - If any Vulkan error is encountered during pipeline creation.
-    /// * [`RendererError`] - If the method is call after [`destroy`] was called.
-    ///
-    /// [`RendererVkContext`]: trait.RendererVkContext.html
-    /// [`RendererError`]: enum.RendererError.html
-    /// [`destroy`]: #method.destroy
-    pub fn set_render_pass<C: RendererVkContext>(
-        &mut self,
-        vk_context: &C,
-        render_pass: vk::RenderPass,
-    ) -> RendererResult<()> {
-        if self.destroyed {
-            return Err(RendererError::Destroyed);
-        }
-
-        unsafe { vk_context.device().destroy_pipeline(self.pipeline, None) };
-        self.pipeline =
-            create_vulkan_pipeline(vk_context.device(), self.pipeline_layout, render_pass)?;
+    pub fn set_render_pass(&mut self, render_pass: vk::RenderPass) -> RendererResult<()> {
+        unsafe { self.device.destroy_pipeline(self.pipeline, None) };
+        self.pipeline = create_vulkan_pipeline(&self.device, self.pipeline_layout, render_pass)?;
         Ok(())
     }
 
@@ -282,50 +314,102 @@ impl Renderer {
         }
     }
 
+    /// Update the fonts texture after having added new fonts to imgui.
+    ///
+    /// # Arguments
+    ///
+    /// * `queue` - A Vulkan queue.
+    ///             It will be used to submit commands during initialization to upload
+    ///             data to the gpu. The type of queue must be supported by the following
+    ///             commands: [vkCmdCopyBufferToImage](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdCopyBufferToImage.html),
+    ///             [vkCmdPipelineBarrier](https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkCmdPipelineBarrier.html)
+    /// * `command_pool` - A Vulkan command pool used to allocate command buffers to upload textures to the gpu.
+    /// * `imgui` - The imgui context.
+    ///
+    /// # Errors
+    ///
+    /// * [`RendererError`] - If any error is encountered during texture update.
+    pub fn update_fonts_texture(
+        &mut self,
+        queue: vk::Queue,
+        command_pool: vk::CommandPool,
+        imgui: &mut Context,
+    ) -> RendererResult<()> {
+        // Generate the new fonts texture
+        let fonts_texture = {
+            let mut fonts = imgui.fonts();
+            let atlas_texture = fonts.build_rgba32_texture();
+
+            Texture::from_rgba8(
+                &self.device,
+                queue,
+                command_pool,
+                &mut self.allocator,
+                atlas_texture.width,
+                atlas_texture.height,
+                atlas_texture.data,
+            )?
+        };
+
+        let mut fonts = imgui.fonts();
+        fonts.tex_id = TextureId::from(usize::MAX);
+
+        // Free Descriptor set the create a new one
+        let old_descriptor_set = self.descriptor_set;
+        unsafe {
+            self.device
+                .free_descriptor_sets(self.descriptor_pool, &[old_descriptor_set])?
+        };
+        self.descriptor_set = create_vulkan_descriptor_set(
+            &self.device,
+            self.descriptor_set_layout,
+            self.descriptor_pool,
+            fonts_texture.image_view,
+            fonts_texture.sampler,
+        )?;
+
+        // Free old fonts texture
+        let mut old_texture = self.fonts_texture.replace(fonts_texture);
+        if let Some(texture) = old_texture.take() {
+            texture.destroy(&self.device, &mut self.allocator)?;
+        }
+
+        Ok(())
+    }
+
     /// Record commands required to render the gui.RendererError.
     ///
     /// # Arguments
     ///
-    /// * `vk_context` - A reference to a type implementing the [`RendererVkContext`] trait.
     /// * `command_buffer` - The Vulkan command buffer that command will be recorded to.
     /// * `draw_data` - A reference to the imgui `DrawData` containing rendering data.
     ///
     /// # Errors
     ///
     /// * [`RendererError`] - If any Vulkan error is encountered during command recording.
-    /// * [`RendererError`] - If the method is call after [`destroy`] was called.
-    ///
-    /// [`RendererVkContext`]: trait.RendererVkContext.html
-    /// [`RendererError`]: enum.RendererError.html
-    /// [`destroy`]: #method.destroy
-    pub fn cmd_draw<C: RendererVkContext>(
+    pub fn cmd_draw(
         &mut self,
-        vk_context: &C,
         command_buffer: vk::CommandBuffer,
         draw_data: &DrawData,
     ) -> RendererResult<()> {
-        if self.destroyed {
-            return Err(RendererError::Destroyed);
-        }
-
         if draw_data.total_vtx_count == 0 {
             return Ok(());
         }
 
         if self.frames.is_none() {
             self.frames.replace(Frames::new(
-                vk_context,
-                &self.allocator,
+                &self.device,
+                &mut self.allocator,
                 draw_data,
                 self.in_flight_frames,
             )?);
         }
 
         let mesh = self.frames.as_mut().unwrap().next();
-        mesh.update(vk_context, &self.allocator, draw_data)?;
+        mesh.update(&self.device, &mut self.allocator, draw_data)?;
 
         unsafe {
-            vk_context.device().cmd_bind_pipeline(
+            self.device.cmd_bind_pipeline(
                 command_buffer,
                 vk::PipelineBindPoint::GRAPHICS,
                 self.pipeline,
@@ -341,11 +425,7 @@ impl Renderer {
             ..Default::default()
         }];
 
-        unsafe {
-            vk_context
-                .device()
-                .cmd_set_viewport(command_buffer, 0, &viewports)
-        };
+        unsafe { self.device.cmd_set_viewport(command_buffer, 0, &viewports) };
 
         // Ortho projection
         let projection = orthographic_vk(
@@ -358,7 +438,7 @@ impl Renderer {
         );
         unsafe {
             let push = any_as_u8_slice(&projection);
-            vk_context.device().cmd_push_constants(
+            self.device.cmd_push_constants(
                 command_buffer,
                 self.pipeline_layout,
                 vk::ShaderStageFlags::VERTEX,
@@ -368,7 +448,7 @@ impl Renderer {
         };
 
         unsafe {
-            vk_context.device().cmd_bind_index_buffer(
+            self.device.cmd_bind_index_buffer(
                 command_buffer,
                 mesh.indices,
                 0,
@@ -377,8 +457,7 @@ impl Renderer {
         };
 
         unsafe {
-            vk_context
-                .device()
+            self.device
                 .cmd_bind_vertex_buffers(command_buffer, 0, &[mesh.vertices], &[0])
         };
 
@@ -416,15 +495,13 @@ impl Renderer {
                                     height: clip_h as _,
                                 },
                             }];
-                            vk_context
-                                .device()
-                                .cmd_set_scissor(command_buffer, 0, &scissors);
+                            self.device.cmd_set_scissor(command_buffer, 0, &scissors);
                         }
 
                         if Some(texture_id) != current_texture_id {
                             let descriptor_set = self.lookup_descriptor_set(texture_id)?;
                             unsafe {
-                                vk_context.device().cmd_bind_descriptor_sets(
+                                self.device.cmd_bind_descriptor_sets(
                                     command_buffer,
                                     vk::PipelineBindPoint::GRAPHICS,
                                     self.pipeline_layout,
@@ -437,7 +514,7 @@ impl Renderer {
                         }
 
                         unsafe {
-                            vk_context.device().cmd_draw_indexed(
+                            self.device.cmd_draw_indexed(
                                 command_buffer,
                                 count as _,
                                 1,
@@ -447,7 +524,12 @@ impl Renderer {
                             )
                         };
                     }
-                    _ => (), // Ignored for now
+                    DrawCmd::ResetRenderState => {
+                        log::trace!("Reset render state command not yet supported")
+                    }
+                    DrawCmd::RawCallback { .. } => {
+                        log::trace!("Raw callback command not yet supported")
+                    }
                 }
             }
 
@@ -457,39 +539,29 @@ impl Renderer {
 
         Ok(())
     }
+}
 
-    /// Destroy Vulkan resources held by the renderer.
-    ///
-    /// # Arguments
-    ///
-    /// * `vk_context` - A reference to a type implementing the [`RendererVkContext`] trait.
-    ///
-    /// # Errors
-    ///
-    /// * [`RendererError`] - If the method is call after [`destroy`] was called.
-    ///
-    /// [`destroy`]: #method.destroy
-    /// [`RendererVkContext`]: trait.RendererVkContext.html
-    /// [`RendererError`]: enum.RendererError.html
-    pub fn destroy<C: RendererVkContext>(&mut self, context: &C) -> RendererResult<()> {
-        if self.destroyed {
-            return Err(RendererError::Destroyed);
-        }
+impl Drop for Renderer {
+    fn drop(&mut self) {
+        log::debug!("Destroying ImGui Renderer");
+        let device = &self.device;
 
         unsafe {
-            let device = context.device();
-            if let Some(mut frames) = self.frames.take() {
-                frames.destroy(context, &self.allocator)?;
+            if let Some(frames) = self.frames.take() {
+                frames
+                    .destroy(device, &mut self.allocator)
+                    .expect("Failed to destroy frame data");
             }
             device.destroy_pipeline(self.pipeline, None);
             device.destroy_pipeline_layout(self.pipeline_layout, None);
             device.destroy_descriptor_pool(self.descriptor_pool, None);
-            self.fonts_texture.destroy(context, &self.allocator)?;
+            self.fonts_texture
+                .take()
+                .unwrap()
+                .destroy(device, &mut self.allocator)
+                .expect("Failed to fronts data");
             device.destroy_descriptor_set_layout(self.descriptor_set_layout, None);
         }
-        self.destroyed = true;
-
-        Ok(())
     }
 }
 
@@ -501,14 +573,14 @@ struct Frames {
 }
 
 impl Frames {
-    fn new<C: RendererVkContext>(
-        vk_context: &C,
-        allocator: &Allocator,
+    fn new(
+        device: &Device,
+        allocator: &mut Allocator,
         draw_data: &DrawData,
         count: usize,
     ) -> RendererResult<Self> {
         let meshes = (0..count)
-            .map(|_| Mesh::new(vk_context, allocator, draw_data))
+            .map(|_| Mesh::new(device, allocator, draw_data))
             .collect::<Result<Vec<_>, _>>()?;
         Ok(Self {
             index: 0,
@@ -523,25 +595,20 @@ impl Frames {
         result
     }
 
-    fn destroy<C: RendererVkContext>(
-        &mut self,
-        vk_context: &C,
-        allocator: &Allocator,
-    ) -> RendererResult<()> {
-        for mesh in self.meshes.iter_mut() {
-            mesh.destroy(vk_context, allocator)?;
+    fn destroy(self, device: &Device, allocator: &mut Allocator) -> RendererResult<()> {
+        for mesh in self.meshes.into_iter() {
+            mesh.destroy(device, allocator)?;
         }
-        self.meshes.clear();
         Ok(())
     }
 }
 
 mod mesh {
 
-    use super::allocator::{Allocator, AllocatorTrait, Memory};
-    use super::{vulkan::*, RendererVkContext};
+    use super::allocator::{Allocate, Allocator, Memory};
+    use super::vulkan::*;
     use crate::RendererResult;
-    use ash::vk;
+    use ash::{vk, Device};
     use imgui::{DrawData, DrawVert};
     use std::mem::size_of;
 
@@ -556,9 +623,9 @@ mod mesh {
     }
 
     impl Mesh {
-        pub fn new<C: RendererVkContext>(
-            vk_context: &C,
-            allocator: &Allocator,
+        pub fn new(
+            device: &Device,
+            allocator: &mut Allocator,
             draw_data: &DrawData,
         ) -> RendererResult<Self> {
             let vertices = create_vertices(draw_data);
@@ -568,7 +635,7 @@ mod mesh {
 
             // Create a vertex buffer
             let (vertices, vertices_mem) = create_and_fill_buffer(
-                vk_context,
+                device,
                 allocator,
                 &vertices,
                 vk::BufferUsageFlags::VERTEX_BUFFER,
@@ -576,7 +643,7 @@ mod mesh {
 
             // Create an index buffer
             let (indices, indices_mem) = create_and_fill_buffer(
-                vk_context,
+                device,
                 allocator,
                 &indices,
                 vk::BufferUsageFlags::INDEX_BUFFER,
@@ -592,75 +659,58 @@ mod mesh {
             })
         }
 
-        pub fn update<C: RendererVkContext>(
+        pub fn update(
             &mut self,
-            vk_context: &C,
-            allocator: &Allocator,
+            device: &Device,
+            allocator: &mut Allocator,
             draw_data: &DrawData,
         ) -> RendererResult<()> {
             let vertices = create_vertices(draw_data);
             if draw_data.total_vtx_count as usize > self.vertex_count {
                 log::trace!("Resizing vertex buffers");
-                self.destroy_vertices(vk_context, allocator)?;
+
                 let vertex_count = vertices.len();
                 let size = vertex_count * size_of::<DrawVert>();
-                let (vertices, vertices_mem) = allocator.create_buffer(
-                    vk_context,
-                    size,
-                    vk::BufferUsageFlags::VERTEX_BUFFER,
-                )?;
+                let (vertices, vertices_mem) =
+                    allocator.create_buffer(device, size, vk::BufferUsageFlags::VERTEX_BUFFER)?;
 
-                self.vertices = vertices;
-                self.vertices_mem = vertices_mem;
                 self.vertex_count = vertex_count;
+
+                let old_vertices = self.vertices;
+                self.vertices = vertices;
+
+                let old_vertices_mem = std::mem::replace(&mut self.vertices_mem, vertices_mem);
+
+                allocator.destroy_buffer(device, old_vertices, old_vertices_mem)?;
             }
-            allocator.update_buffer(vk_context, &self.vertices_mem, &vertices)?;
+            allocator.update_buffer(device, &self.vertices_mem, &vertices)?;
 
             let indices = create_indices(draw_data);
             if draw_data.total_idx_count as usize > self.index_count {
                 log::trace!("Resizing index buffers");
-                self.destroy_indices(vk_context, allocator)?;
+
                 let index_count = indices.len();
                 let size = index_count * size_of::<u16>();
-                let (indices, indices_mem) = allocator.create_buffer(
-                    vk_context,
-                    size,
-                    vk::BufferUsageFlags::INDEX_BUFFER,
-                )?;
-                self.indices = indices;
-                self.indices_mem = indices_mem;
+                let (indices, indices_mem) =
+                    allocator.create_buffer(device, size, vk::BufferUsageFlags::INDEX_BUFFER)?;
+
                 self.index_count = index_count;
+
+                let old_indices = self.indices;
+                self.indices = indices;
+
+                let old_indices_mem = std::mem::replace(&mut self.indices_mem, indices_mem);
+
+                allocator.destroy_buffer(device, old_indices, old_indices_mem)?;
             }
-            allocator.update_buffer(vk_context, &self.indices_mem, &indices)?;
+            allocator.update_buffer(device, &self.indices_mem, &indices)?;
 
             Ok(())
         }
 
-        pub fn destroy<C: RendererVkContext>(
-            &mut self,
-            vk_context: &C,
-            allocator: &Allocator,
-        ) -> RendererResult<()> {
-            self.destroy_indices(vk_context, allocator)?;
-            self.destroy_vertices(vk_context, allocator)?;
-            Ok(())
-        }
-
-        fn destroy_vertices<C: RendererVkContext>(
-            &mut self,
-            vk_context: &C,
-            allocator: &Allocator,
-        ) -> RendererResult<()> {
-            allocator.destroy_buffer(vk_context, self.vertices, &self.vertices_mem)?;
-            Ok(())
-        }
-
-        fn destroy_indices<C: RendererVkContext>(
-            &mut self,
-            vk_context: &C,
-            allocator: &Allocator,
-        ) -> RendererResult<()> {
-            allocator.destroy_buffer(vk_context, self.indices, &self.indices_mem)?;
+        pub fn destroy(self, device: &Device, allocator: &mut Allocator) -> RendererResult<()> {
+            allocator.destroy_buffer(device, self.vertices, self.vertices_mem)?;
+            allocator.destroy_buffer(device, self.indices, self.indices_mem)?;
             Ok(())
         }
     }
